'''
Created on 15.2.2011

@author: jan.vlcinsky@gmail.com
'''
import unittest
from basex import BaseXClient

class Test(unittest.TestCase):
    def documentExists(self, docName):
        '''
        docName shall include collection name e.g.: 'db/path/to/dir/file.xml'
        '''
        command = "xquery count(db:open('%s'))" % docName
        res = self.session.execute(command)
        print res
        return int(res) > 0
    
    def getQueryResults(self, query):
        items = []
        while query.more():
            items.append(query.next())
        return items
    
    def getCollections(self):
        query = self.session.query("db:list()")
        query.init()
        cols = self.getQueryResults(query)
        query.close()
        return cols    

    def getDocuments(self):
        query = self.session.query("db:list()")
        query.init()
        cols = self.getQueryResults(query)
        query.close()
        return cols    

    def setUp(self):
        self.session = BaseXClient.Session('localhost', 1984, 'admin', 'admin')
        if not "testDB2" in self.getCollections():
            self.session.execute("create db testDB2")


    def tearDown(self):
        #if "testDB2" in self.getCollections():
        #    self.session.execute("drop db testDB2")
        self.session.close()


    def test_simpleQueryExecute(self):
        res = self.session.execute("xquery 1 to 5")
        self.assertEqual(res, "1 2 3 4 5")

    def test_simpleQueryExecuteTwice(self):
        res = self.session.execute("xquery 1 to 5")
        self.assertEqual(res, "1 2 3 4 5")
        res = self.session.execute("xquery 1 to 5")
        self.assertEqual(res, "1 2 3 4 5")

    def test_badCommand(self):
        res = self.session.execute("xquery 1 to 5")
        self.assertEqual(res, "1 2 3 4 5")
        #now we fail
        self.assertRaises(IOError, self.session.execute,"SQLxquery 1 to 5")
        #and we shall work properly after that
        res = self.session.execute("xquery 1 to 3")
        self.assertEqual(res, "1 2 3")
        
        
    def test_sequetialQueryRead(self):
        # create query instance
        input = "for $i in 1 to 3 return <xml>Text {$i}</xml>"
        query = self.session.query(input)
        # initialize query
        query.init()
        # loop through all results
        items = self.getQueryResults(query)
        expected_items = ["<xml>Text %d</xml>" % i for i in range(1,4)]
        query.close()
        self.assertEqual(items, expected_items)

    def test_queryBind(self):
        input = "declare variable $name external; for $i in 1 to 10 return element { $name } { $i }"
        query = self.session.query(input)
        # bind variable
        query.bind("$name", "number")
        # initialize query
        query.init()
        # loop through all results
        items = self.getQueryResults(query)
        # close query object  
        query.close()
        expected_items = ["<number>%d</number>" % i for i in range(1,11)]
        self.assertEqual(items, expected_items)

    def test_queryBindMulti(self):
        input = '''
        declare variable $name external := 'nomen' ;
        declare variable $att_val external := '15';
        for $i in 1 to 10 return 
        element { $name } {
            attribute att { $att_val},
             $i
        }
         '''
        query = self.session.query(input)
        # bind variable
        query.bind("$name", "number")
        query.bind("$att_val", "123456")
        # initialize query
        query.init()
        # loop through all results
        items = self.getQueryResults(query)
        # close query object  
        query.close()
        expected_items = ['<number att="123456">%d</number>' % i for i in range(1,11)]
        self.assertEqual(items, expected_items)

    def test_addDocument(self):
        self.session.execute("open testDB2")
        self.session.add("World.xml", "", "<x>Hello World!</x>")
        self.assertTrue(self.documentExists("testDB2/World.xml"))
        #Here one intentional failure
        self.assertRaises(IOError, self.session.add ,"World.xml", "small", "<x>Hello Big World!</xxx>")
        #We shall survive and continue
        self.session.add("World.xml", "big", "<x>Hello Big World!</x>")
        self.assertTrue(self.documentExists("testDB2/big/World.xml"))
        self.session.add("World.xml", "small", "<x>Hello Big World!</x>")
        self.assertTrue(self.documentExists("testDB2/small/World.xml"))
        self.session.execute("close")

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()